@@ -13,17 +13,30 @@
 #   under the License.
 #
 
-import copy
-
 from keystoneauth1 import exceptions as ks_exc
+from osc_lib import exceptions
 
 from openstackclient.identity.v2_0 import project
-from openstackclient.tests import fakes
 from openstackclient.tests.identity.v2_0 import fakes as identity_fakes
 
 
 class TestProject(identity_fakes.TestIdentityv2):
 
+    fake_project = identity_fakes.FakeProject.create_one_project()
+
+    columns = (
+        'description',
+        'enabled',
+        'id',
+        'name',
+    )
+    datalist = (
+        fake_project.description,
+        True,
+        fake_project.id,
+        fake_project.name,
+    )
+
     def setUp(self):
         super(TestProject, self).setUp()
 
@@ -34,39 +47,22 @@ class TestProject(identity_fakes.TestIdentityv2):
 
 class TestProjectCreate(TestProject):
 
-    columns = (
-        'description',
-        'enabled',
-        'id',
-        'name',
-    )
-    datalist = (
-        identity_fakes.project_description,
-        True,
-        identity_fakes.project_id,
-        identity_fakes.project_name,
-    )
-
     def setUp(self):
         super(TestProjectCreate, self).setUp()
 
-        self.projects_mock.create.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(identity_fakes.PROJECT),
-            loaded=True,
-        )
+        self.projects_mock.create.return_value = self.fake_project
 
         # Get the command object to test
         self.cmd = project.CreateProject(self.app, None)
 
     def test_project_create_no_options(self):
         arglist = [
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
             ('enable', False),
             ('disable', False),
-            ('name', identity_fakes.project_name),
+            ('name', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -81,7 +77,7 @@ class TestProjectCreate(TestProject):
             'enabled': True,
         }
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
+            self.fake_project.name,
             **kwargs
         )
         self.assertEqual(self.columns, columns)
@@ -90,11 +86,11 @@ class TestProjectCreate(TestProject):
     def test_project_create_description(self):
         arglist = [
             '--description', 'new desc',
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
             ('description', 'new desc'),
-            ('name', identity_fakes.project_name),
+            ('name', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -109,7 +105,7 @@ class TestProjectCreate(TestProject):
             'enabled': True,
         }
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
+            self.fake_project.name,
             **kwargs
         )
 
@@ -119,12 +115,12 @@ class TestProjectCreate(TestProject):
     def test_project_create_enable(self):
         arglist = [
             '--enable',
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
             ('enable', True),
             ('disable', False),
-            ('name', identity_fakes.project_name),
+            ('name', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -139,7 +135,7 @@ class TestProjectCreate(TestProject):
             'enabled': True,
         }
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
+            self.fake_project.name,
             **kwargs
         )
 
@@ -149,12 +145,12 @@ class TestProjectCreate(TestProject):
     def test_project_create_disable(self):
         arglist = [
             '--disable',
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
             ('enable', False),
             ('disable', True),
-            ('name', identity_fakes.project_name),
+            ('name', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -169,7 +165,7 @@ class TestProjectCreate(TestProject):
             'enabled': False,
         }
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
+            self.fake_project.name,
             **kwargs
         )
 
@@ -180,11 +176,11 @@ class TestProjectCreate(TestProject):
         arglist = [
             '--property', 'fee=fi',
             '--property', 'fo=fum',
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
             ('property', {'fee': 'fi', 'fo': 'fum'}),
-            ('name', identity_fakes.project_name),
+            ('name', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -201,7 +197,7 @@ class TestProjectCreate(TestProject):
             'fo': 'fum',
         }
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
+            self.fake_project.name,
             **kwargs
         )
 
@@ -215,19 +211,15 @@ class TestProjectCreate(TestProject):
         # need to make this throw an exception...
         self.projects_mock.create.side_effect = _raise_conflict
 
-        self.projects_mock.get.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(identity_fakes.PROJECT),
-            loaded=True,
-        )
+        self.projects_mock.get.return_value = self.fake_project
 
         arglist = [
             '--or-show',
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
-            ('name', identity_fakes.project_name),
             ('or_show', True),
+            ('name', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -237,7 +229,7 @@ class TestProjectCreate(TestProject):
         columns, data = self.cmd.take_action(parsed_args)
 
         # ProjectManager.create(name, description, enabled)
-        self.projects_mock.get.assert_called_with(identity_fakes.project_name)
+        self.projects_mock.get.assert_called_with(self.fake_project.name)
 
         # Set expected values
         kwargs = {
@@ -245,7 +237,7 @@ class TestProjectCreate(TestProject):
             'enabled': True,
         }
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
+            self.fake_project.name,
             **kwargs
         )
 
@@ -255,11 +247,11 @@ class TestProjectCreate(TestProject):
     def test_project_create_or_show_not_exists(self):
         arglist = [
             '--or-show',
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
-            ('name', identity_fakes.project_name),
             ('or_show', True),
+            ('name', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -274,7 +266,7 @@ class TestProjectCreate(TestProject):
             'enabled': True,
         }
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
+            self.fake_project.name,
             **kwargs
         )
 
@@ -288,11 +280,7 @@ class TestProjectDelete(TestProject):
         super(TestProjectDelete, self).setUp()
 
         # This is the return value for utils.find_resource()
-        self.projects_mock.get.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(identity_fakes.PROJECT),
-            loaded=True,
-        )
+        self.projects_mock.get.return_value = self.fake_project
         self.projects_mock.delete.return_value = None
 
         # Get the command object to test
@@ -300,17 +288,17 @@ class TestProjectDelete(TestProject):
 
     def test_project_delete_no_options(self):
         arglist = [
-            identity_fakes.project_id,
+            self.fake_project.id,
         ]
         verifylist = [
-            ('projects', [identity_fakes.project_id]),
+            ('projects', [self.fake_project.id]),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.take_action(parsed_args)
 
         self.projects_mock.delete.assert_called_with(
-            identity_fakes.project_id,
+            self.fake_project.id,
         )
         self.assertIsNone(result)
 
@@ -320,13 +308,7 @@ class TestProjectList(TestProject):
     def setUp(self):
         super(TestProjectList, self).setUp()
 
-        self.projects_mock.list.return_value = [
-            fakes.FakeResource(
-                None,
-                copy.deepcopy(identity_fakes.PROJECT),
-                loaded=True,
-            ),
-        ]
+        self.projects_mock.list.return_value = [self.fake_project]
 
         # Get the command object to test
         self.cmd = project.ListProject(self.app, None)
@@ -345,8 +327,8 @@ class TestProjectList(TestProject):
         collist = ('ID', 'Name')
         self.assertEqual(collist, columns)
         datalist = ((
-            identity_fakes.project_id,
-            identity_fakes.project_name,
+            self.fake_project.id,
+            self.fake_project.name,
         ), )
         self.assertEqual(datalist, tuple(data))
 
@@ -368,9 +350,9 @@ class TestProjectList(TestProject):
         collist = ('ID', 'Name', 'Description', 'Enabled')
         self.assertEqual(collist, columns)
         datalist = ((
-            identity_fakes.project_id,
-            identity_fakes.project_name,
-            identity_fakes.project_description,
+            self.fake_project.id,
+            self.fake_project.name,
+            self.fake_project.description,
             True,
         ), )
         self.assertEqual(datalist, tuple(data))
@@ -381,26 +363,18 @@ class TestProjectSet(TestProject):
     def setUp(self):
         super(TestProjectSet, self).setUp()
 
-        self.projects_mock.get.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(identity_fakes.PROJECT),
-            loaded=True,
-        )
-        self.projects_mock.update.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(identity_fakes.PROJECT),
-            loaded=True,
-        )
+        self.projects_mock.get.return_value = self.fake_project
+        self.projects_mock.update.return_value = self.fake_project
 
         # Get the command object to test
         self.cmd = project.SetProject(self.app, None)
 
     def test_project_set_no_options(self):
         arglist = [
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
-            ('project', identity_fakes.project_name),
+            ('project', self.fake_project.name),
             ('enable', False),
             ('disable', False),
         ]
@@ -410,16 +384,36 @@ class TestProjectSet(TestProject):
 
         self.assertIsNone(result)
 
-    def test_project_set_name(self):
+    def test_project_set_unexist_project(self):
         arglist = [
-            '--name', 'qwerty',
-            identity_fakes.project_name,
+            "unexist-project",
         ]
         verifylist = [
-            ('name', 'qwerty'),
+            ('project', "unexist-project"),
+            ('name', None),
+            ('description', None),
             ('enable', False),
             ('disable', False),
-            ('project', identity_fakes.project_name),
+            ('property', None),
+        ]
+        self.projects_mock.get.side_effect = exceptions.NotFound(None)
+        self.projects_mock.find.side_effect = exceptions.NotFound(None)
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(
+            exceptions.CommandError, self.cmd.take_action, parsed_args)
+
+    def test_project_set_name(self):
+        arglist = [
+            '--name', self.fake_project.name,
+            self.fake_project.name,
+        ]
+        verifylist = [
+            ('name', self.fake_project.name),
+            ('enable', False),
+            ('disable', False),
+            ('project', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -427,26 +421,26 @@ class TestProjectSet(TestProject):
 
         # Set expected values
         kwargs = {
-            'description': identity_fakes.project_description,
+            'description': self.fake_project.description,
             'enabled': True,
-            'tenant_name': 'qwerty',
+            'tenant_name': self.fake_project.name,
         }
         self.projects_mock.update.assert_called_with(
-            identity_fakes.project_id,
+            self.fake_project.id,
             **kwargs
         )
         self.assertIsNone(result)
 
     def test_project_set_description(self):
         arglist = [
-            '--description', 'new desc',
-            identity_fakes.project_name,
+            '--description', self.fake_project.description,
+            self.fake_project.name,
         ]
         verifylist = [
-            ('description', 'new desc'),
+            ('description', self.fake_project.description),
             ('enable', False),
             ('disable', False),
-            ('project', identity_fakes.project_name),
+            ('project', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -454,12 +448,12 @@ class TestProjectSet(TestProject):
 
         # Set expected values
         kwargs = {
-            'description': 'new desc',
+            'description': self.fake_project.description,
             'enabled': True,
-            'tenant_name': identity_fakes.project_name,
+            'tenant_name': self.fake_project.name,
         }
         self.projects_mock.update.assert_called_with(
-            identity_fakes.project_id,
+            self.fake_project.id,
             **kwargs
         )
         self.assertIsNone(result)
@@ -467,12 +461,12 @@ class TestProjectSet(TestProject):
     def test_project_set_enable(self):
         arglist = [
             '--enable',
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
             ('enable', True),
             ('disable', False),
-            ('project', identity_fakes.project_name),
+            ('project', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -480,12 +474,12 @@ class TestProjectSet(TestProject):
 
         # Set expected values
         kwargs = {
-            'description': identity_fakes.project_description,
+            'description': self.fake_project.description,
             'enabled': True,
-            'tenant_name': identity_fakes.project_name,
+            'tenant_name': self.fake_project.name,
         }
         self.projects_mock.update.assert_called_with(
-            identity_fakes.project_id,
+            self.fake_project.id,
             **kwargs
         )
         self.assertIsNone(result)
@@ -493,12 +487,12 @@ class TestProjectSet(TestProject):
     def test_project_set_disable(self):
         arglist = [
             '--disable',
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
             ('enable', False),
             ('disable', True),
-            ('project', identity_fakes.project_name),
+            ('project', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -506,12 +500,12 @@ class TestProjectSet(TestProject):
 
         # Set expected values
         kwargs = {
-            'description': identity_fakes.project_description,
+            'description': self.fake_project.description,
             'enabled': False,
-            'tenant_name': identity_fakes.project_name,
+            'tenant_name': self.fake_project.name,
         }
         self.projects_mock.update.assert_called_with(
-            identity_fakes.project_id,
+            self.fake_project.id,
             **kwargs
         )
         self.assertIsNone(result)
@@ -520,11 +514,11 @@ class TestProjectSet(TestProject):
         arglist = [
             '--property', 'fee=fi',
             '--property', 'fo=fum',
-            identity_fakes.project_name,
+            self.fake_project.name,
         ]
         verifylist = [
             ('property', {'fee': 'fi', 'fo': 'fum'}),
-            ('project', identity_fakes.project_name),
+            ('project', self.fake_project.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -532,14 +526,14 @@ class TestProjectSet(TestProject):
 
         # Set expected values
         kwargs = {
-            'description': identity_fakes.project_description,
+            'description': self.fake_project.description,
             'enabled': True,
-            'tenant_name': identity_fakes.project_name,
+            'tenant_name': self.fake_project.name,
             'fee': 'fi',
             'fo': 'fum',
         }
         self.projects_mock.update.assert_called_with(
-            identity_fakes.project_id,
+            self.fake_project.id,
             **kwargs
         )
         self.assertIsNone(result)
@@ -547,24 +541,22 @@ class TestProjectSet(TestProject):
 
 class TestProjectShow(TestProject):
 
+    fake_proj_show = identity_fakes.FakeProject.create_one_project()
+
     def setUp(self):
         super(TestProjectShow, self).setUp()
 
-        self.projects_mock.get.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(identity_fakes.PROJECT),
-            loaded=True,
-        )
+        self.projects_mock.get.return_value = self.fake_proj_show
 
         # Get the command object to test
         self.cmd = project.ShowProject(self.app, None)
 
     def test_project_show(self):
         arglist = [
-            identity_fakes.project_id,
+            self.fake_proj_show.id,
         ]
         verifylist = [
-            ('project', identity_fakes.project_id),
+            ('project', self.fake_proj_show.id),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -573,16 +565,16 @@ class TestProjectShow(TestProject):
         # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
         self.projects_mock.get.assert_called_with(
-            identity_fakes.project_id,
+            self.fake_proj_show.id,
         )
 
         collist = ('description', 'enabled', 'id', 'name', 'properties')
         self.assertEqual(collist, columns)
         datalist = (
-            identity_fakes.project_description,
+            self.fake_proj_show.description,
             True,
-            identity_fakes.project_id,
-            identity_fakes.project_name,
+            self.fake_proj_show.id,
+            self.fake_proj_show.name,
             '',
         )
         self.assertEqual(datalist, data)
@@ -590,25 +582,35 @@ class TestProjectShow(TestProject):
 
 class TestProjectUnset(TestProject):
 
+    attr = {'fee': 'fi', 'fo': 'fum'}
+    fake_proj = identity_fakes.FakeProject.create_one_project(attr)
+
     def setUp(self):
         super(TestProjectUnset, self).setUp()
 
-        project_dict = {'fee': 'fi', 'fo': 'fum'}
-        project_dict.update(identity_fakes.PROJECT)
-        self.projects_mock.get.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(project_dict),
-            loaded=True,
-        )
+        self.projects_mock.get.return_value = self.fake_proj
 
         # Get the command object to test
         self.cmd = project.UnsetProject(self.app, None)
 
+    def test_project_unset_no_options(self):
+        arglist = [
+            self.fake_proj.name,
+        ]
+        verifylist = [
+            ('project', self.fake_proj.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertIsNone(result)
+
     def test_project_unset_key(self):
         arglist = [
             '--property', 'fee',
             '--property', 'fo',
-            identity_fakes.project_name,
+            self.fake_proj.name,
         ]
         verifylist = [
             ('property', ['fee', 'fo']),
@@ -618,16 +620,16 @@ class TestProjectUnset(TestProject):
         result = self.cmd.take_action(parsed_args)
         # Set expected values
         kwargs = {
-            'description': identity_fakes.project_description,
+            'description': self.fake_proj.description,
             'enabled': True,
             'fee': None,
             'fo': None,
-            'id': identity_fakes.project_id,
-            'name': identity_fakes.project_name,
+            'id': self.fake_proj.id,
+            'name': self.fake_proj.name,
         }
 
         self.projects_mock.update.assert_called_with(
-            identity_fakes.project_id,
+            self.fake_proj.id,
             **kwargs
         )
         self.assertIsNone(result)
